﻿using UnityEngine;
using System.Collections;

public class Event : MonoBehaviour 
{
    //max time to fix the event in sec
    public float timeAllowed = 10.0f;
    public float timeToStart = 10.0f;

    //if null no item is required
    public Item requiredItem;

   [HideInInspector] 
   public bool alreadyFixedOrMissed = false;

   private GameObject eventRenderer;
   private GameObject eventFixedRenderer;

   private GameManager gm;

   private float elapsedTime;
   private bool isStarted;

   bool playerInTrigger;

	void Start () 
    {
        gm = GameManager.Instance;
        isStarted = false;
        elapsedTime = 0f;
        playerInTrigger = false;
	}
	
	void Update () 
    {
        if (gm.currentGameState == EGameState.GS_GAME)
        {
            if (!isStarted)
            {
                if (gm.elapsedTimeInGame >= timeToStart)
                {
                    isStarted = true;
                    Activate();
                }
            }
            else if(!alreadyFixedOrMissed)
            {
                if(playerInTrigger && Input.GetKey(KeyCode.Space))
                {
                     if (gm.elapsedTimeForCurrentEvent <= timeAllowed)
                {
                    if (requiredItem)
                    {
                         if(GameObject.FindGameObjectWithTag("Player").GetComponent<PlayerInventory>().Contains(requiredItem))
                         {
                             Debug.Log("Required Item ok");
                             Fixed();
                             gm.EventFixed(elapsedTime, timeAllowed);
                             alreadyFixedOrMissed = true;
                         }
                    }
                    else
                    {
                        Debug.Log("No item required");
                        Fixed();
                        gm.EventFixed(elapsedTime, timeAllowed);
                        alreadyFixedOrMissed = true;
                    }
                }
                }
                elapsedTime += Time.deltaTime;
                if (elapsedTime >= timeAllowed)
                {
                    Debug.Log("Elapsed Time for event " + name + " : " + elapsedTime);
                    collider.enabled = false;
                    alreadyFixedOrMissed = true;
                    gm.EventMissed();
                }    
            }
        }
	}

    void OnTriggerEnter(Collider other)
    {
        if (other.CompareTag("Player"))
        {
            gm.SaySomething();
            playerInTrigger = true;
        }
    }

    void OnTriggerExit(Collider other)
    {
        if (other.CompareTag("Player"))
        {
            playerInTrigger = false;
        }
    }

   /* void OnTriggerStay(Collider other)
    {
        if (other.CompareTag("Player"))
        {
            Debug.Log("TEST");
            if (Input.GetKeyUp(KeyCode.Space))
            {
                Debug.Log("TEST SPACE");
                if (gm.elapsedTimeForCurrentEvent <= timeAllowed)
                {
                    if (requiredItem)
                    {
                         if(other.GetComponent<PlayerInventory>().Contains(requiredItem))
                         {
                             Debug.Log("Required Item ok");
                             Fixed();
                             gm.EventFixed(elapsedTime, timeAllowed);
                             alreadyFixedOrMissed = true;
                         }
                    }
                    else
                    {
                        Debug.Log("No item required");
                        Fixed();
                        gm.EventFixed(elapsedTime, timeAllowed);
                        alreadyFixedOrMissed = true;
                    }
                }
                
            }
        }
    }*/


    public virtual void Activate()
    {
        collider.enabled = true;
        gm.SaySomething();
    }

    public virtual void Fixed()
    {
        collider.enabled = false;

    }
}
