﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Collections.Generic;

public enum EGameState
{
    GS_MENU,
    GS_GAME,
    GS_END
}

public class GameManager : MonoBehaviour 
{
    private static GameManager _instance;
    public static GameManager Instance
    {
        get
        {
            if (_instance == null)
                _instance = GameObject.FindObjectOfType<GameManager>();
            return _instance;
        }
    }

    private List<Event> events;
    private int currentEventIndex;
    public EGameState currentGameState;
    public float elapsedTimeForCurrentEvent;
    private bool eventsStarted;
    private int score;
    private int missedEvents;
    private int fixedEvents;
    private List<Event> allEvents;

    public List<float> timers;
    public float delayBeforeStart = 5f;
    public float allowedMissedEvents = 3;
    public float elapsedTimeInGame;

    public Text ScoreTxt;
    public Text playerExclamation;
    public Text GameOver;

    public List<string> exclamations;

    private CharacControls cc;

    void Awake()
    {
        events = new List<Event>();
    }

	void Start () 
    {
        currentGameState = EGameState.GS_GAME;
        currentEventIndex = 0;
        elapsedTimeForCurrentEvent = 0;
        eventsStarted = false;
        score = 0;
        missedEvents = 0;
        elapsedTimeInGame = 0;
        ScoreTxt.text = "Score: 0";
        cc = GameObject.FindGameObjectWithTag("Player").GetComponent<CharacControls>();

        //build events list
        //@todo call BuildEventList instead
        /*Transform parent = GameObject.Find("Events").transform;
        for (int i = 0; i < parent.childCount; i++)
        {
            events.Add(parent.GetChild(i).GetComponent<Event>());
        }*/
        BuildEventsList();

        //Debug
        foreach (Event e in events)
        {
            Debug.Log(e.name);
        }
	}

    // Randomly build the events list for this game. 
    private void BuildEventsList()
    {
        Transform parent = GameObject.Find("Events").transform;
        allEvents = new List<Event>();
        for (int i = 0; i < parent.childCount; i++)
        {
            allEvents.Add(parent.GetChild(i).GetComponent<Event>());
        }

        int timeIndex = 0;
        while (allEvents.Count > 0)
        {
            int index = Random.Range(0, allEvents.Count);
            Event e = allEvents[index];
            allEvents.Remove(e);
            e.timeToStart = timers[timeIndex];
            events.Add(e);
            timeIndex++;
        }
    }

    //UNUSED
   /* private void CheckSequence(Event e)
    {
        if (e.nextEventInSequence)
        {
           Event next = allEvents.Find(x => x == e.nextEventInSequence);
           allEvents.Remove(next);
           events.Add(next);
           CheckSequence(next);
        }
    }*/
	
	void Update () 
    {
        if (currentGameState == EGameState.GS_MENU)
        {
            Debug.Log("In Main Menu");
        }
        else if (currentGameState == EGameState.GS_GAME)
        {
            elapsedTimeInGame += Time.deltaTime;

            if (missedEvents >= allowedMissedEvents)
            {
                //Lose game
                Debug.Log("LOSE");
                currentGameState = EGameState.GS_END;
                PlayerPrefs.SetInt("EndCondition", 0);
                PlayerPrefs.SetInt("FixedEvents", fixedEvents);
                PlayerPrefs.SetInt("MissedEvents", missedEvents);
                cc.canMove = false;
                
            }
            else if (fixedEvents >= events.Count)
            {
                //Full Win game
                Debug.Log("WIN");
                currentGameState = EGameState.GS_END;
                PlayerPrefs.SetInt("EndCondition", 1);
                PlayerPrefs.SetInt("FixedEvents", fixedEvents);
                PlayerPrefs.SetInt("MissedEvents", missedEvents);
                cc.canMove = false; 
                //StartCoroutine(AnimGameOver());
            }
            else if (fixedEvents + missedEvents == events.Count)
            {
                Debug.Log("Finished");
                currentGameState = EGameState.GS_END;
                PlayerPrefs.SetInt("EndCondition", 2);
                PlayerPrefs.SetInt("FixedEvents", fixedEvents);
                PlayerPrefs.SetInt("MissedEvents", missedEvents);
                cc.canMove = false;
              //  StartCoroutine(AnimGameOver());
            }

            //first event: wait before start
           /* if (currentEventIndex == 0)
            {
                if (!eventsStarted)
                {
                    elapsedTimeForCurrentEvent += Time.deltaTime;
                    if (elapsedTimeForCurrentEvent >= delayBeforeStart)
                    {
                        events[currentEventIndex].Activate();
                        elapsedTimeForCurrentEvent = 0;
                        eventsStarted = true;
                    }
                }
                else
                {
                    RunCurrentEvent();
                }
            }
            else
            {
                RunCurrentEvent();
            }*/
        }
        else if (currentGameState == EGameState.GS_END)
        {
           // Debug.Log("In End Game Screen");
            StartCoroutine(AnimGameOver());
        }
	}

    private void RunCurrentEvent()
    {
        elapsedTimeForCurrentEvent += Time.deltaTime;
        if (elapsedTimeForCurrentEvent > events[currentEventIndex].timeAllowed)
        {
            currentEventIndex++;
            missedEvents++;
            elapsedTimeForCurrentEvent = 0;
            if (currentEventIndex < events.Count && missedEvents < allowedMissedEvents)
            {
                events[currentEventIndex].Activate();
            }
            else
            {
                DisableAllEvents();
                currentGameState = EGameState.GS_END;
            }
        }
    }

    public void EventFixed(float timeElasped, float timeAllowed)
    {
        /*score += (int)(events[currentEventIndex].timeAllowed - elapsedTimeForCurrentEvent);
        Debug.Log("Score = " + score);

        elapsedTimeForCurrentEvent = 0f;
        currentEventIndex++;
        if (currentEventIndex < events.Count)
        {
            events[currentEventIndex].Activate();
        }
        else
        {
            DisableAllEvents();
            currentGameState = EGameState.GS_END;
        }*/

        score += (int)(timeAllowed - timeElasped);
        ScoreTxt.text = "Score: " + score;
        fixedEvents++;
    }

    public void EventMissed()
    {
        missedEvents++;
    }

    private void DisableAllEvents()
    {
        foreach (Event e in events)
        {
            e.collider.enabled = false;
        }
    }

    public void SaySomething()
    {
        int i = Random.Range(0, exclamations.Count);
        playerExclamation.text = exclamations[i];
        StartCoroutine(ResetText());

    }

    private IEnumerator ResetText()
    {
        yield return new WaitForSeconds(2f);
        playerExclamation.text = "";
    }

    private IEnumerator AnimGameOver()
    {
      //  Debug.Log("ANIM");
        float y = GameOver.rectTransform.anchoredPosition.y;
        if (y > 0)
        {
            //   Debug.Log("y = " + y);
            y -= 10;
            GameOver.rectTransform.anchoredPosition = new Vector3(0, y, 0);
        }
        else
        {
            GameOver.rectTransform.anchoredPosition = Vector3.zero;
            StartCoroutine(ShowEndGame());
            
        }
        yield return null;
    }

    private IEnumerator ShowEndGame()
    {
        yield return new WaitForSeconds(3f);
        Application.LoadLevel("end");
    }
}
